import os
import yaml

from juju.errors import FileNotFound
from juju.charm.provider import get_charm_from_path
from .errors import CharmNotFound, CharmError


class LocalCharmRepository(object):
    """Charm repository in a local directory."""

    def __init__(self, path):
        self.path = path

    def __iter__(self):
        for dentry in os.listdir(self.path):
            dentry_path = os.path.join(self.path, dentry)
            try:
                yield get_charm_from_path(dentry_path)
            except FileNotFound:
                continue
            # there is a broken charm in the repo, but that
            # shouldn't stop us from continuing
            except (CharmError, yaml.YAMLError):
                continue

    def find(self, name):
        """Find a charm with the given name.

        If multiple charms are found with different versions, the most
        recent one (greatest revision) will be returned.
        """
        latest = None
        for charm in self:
            if charm.metadata.name == name:
                if (latest is None or
                    latest.metadata.revision < charm.metadata.revision):
                    latest = charm

        if latest is None:
            raise CharmNotFound(self.path, name)

        return latest
