from juju.lib.testing import TestCase
from juju.charm.base import CharmBase


class MyCharm(CharmBase):
    pass


class CharmBaseTest(TestCase):

    def setUp(self):
        self.charm = MyCharm()

    def assertUnsupported(self, callable, attr_name):
        try:
            callable()
        except NotImplementedError, e:
            self.assertEquals(str(e),
                              "MyCharm.%s not supported" % attr_name)
        else:
            self.fail("MyCharm.%s didn't fail" % attr_name)

    def test_as_bundle_is_unsupported(self):
        self.assertUnsupported(lambda: self.charm.as_bundle(),
                               "as_bundle()")

    def test_compute_sha256_is_unsupported(self):
        self.assertUnsupported(lambda: self.charm.compute_sha256(),
                               "compute_sha256()")

    def test_get_sha256_fails_if_not_preset(self):
        """
        get_sha256() will internally call compute_sha256() if
        the digest value hasn't been previously set.
        """
        self.assertUnsupported(lambda: self.charm.get_sha256(),
                               "compute_sha256()")

    def test_compute_and_cache_sha256(self):
        """
        The value computed by compute_sha256() on a child class
        is returned by get_sha256, and cached permanently.
        """
        sha256 = ["mysha"]

        class CustomCharm(CharmBase):

            def compute_sha256(self):
                return sha256[0]

        charm = CustomCharm()
        self.assertEquals(charm.get_sha256(), "mysha")
        sha256 = ["anothervalue"]
        # Should still be the same, since the old one was cached.
        self.assertEquals(charm.get_sha256(), "mysha")
