from juju.charm.errors import (
    CharmIdError, CharmNotFound, InvalidCharmHook, NewerCharmNotFound)
from juju.errors import CharmError

from juju.lib.testing import TestCase


class CharmErrorsTest(TestCase):

    def test_NewerCharmNotFound(self):
        error = NewerCharmNotFound("local:name:21")
        self.assertEquals(
            str(error),
            "Charm 'local:name:21' is the latest revision known")
        self.assertTrue(isinstance(error, CharmError))

    def test_CharmIdError(self):
        error = CharmIdError("foobar:/adfsa:slashot", "bad magic")
        self.assertEquals(
            str(error),
            "Invalid charm id 'foobar:/adfsa:slashot' bad magic")
        self.assertTrue(isinstance(error, CharmError))

    def test_CharmNotFound(self):
        error = CharmNotFound("/path", "cassandra")
        self.assertEquals(
            str(error),
            "Charm 'cassandra' not found in repository /path")
        self.assertTrue(isinstance(error, CharmError))

    def test_InvalidCharmHook(self):
        error = InvalidCharmHook("mysql", "magic-relation-changed")
        self.assertEquals(
            str(error),
            "Charm 'mysql' does not contain hook 'magic-relation-changed'")
        self.assertTrue(isinstance(error, CharmError))
